package sun.audio;

import javax.sound.midi.MetaMessage;
import javax.sound.midi.MetaEventListener;
import javax.sound.sampled.AudioFormat;
import java.io.BufferedInputStream;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequencer;
import java.io.IOException;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiSystem;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import com.sun.media.sound.DataPusher;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;
import com.sun.media.sound.Toolkit;
import java.io.InputStream;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.Mixer;
import java.util.Vector;
import java.util.Hashtable;

public final class AudioDevice
{
    private boolean DEBUG;
    private Hashtable clipStreams;
    private Vector infos;
    private boolean playing;
    private Mixer mixer;
    public static final AudioDevice device;
    
    private AudioDevice() {
        this.DEBUG = false;
        this.playing = false;
        this.mixer = null;
        this.clipStreams = new Hashtable();
        this.infos = new Vector();
    }
    
    private synchronized void startSampled(AudioInputStream pcmConvertedAudioInputStream, InputStream inputStream) throws UnsupportedAudioFileException, LineUnavailableException {
        pcmConvertedAudioInputStream = Toolkit.getPCMConvertedAudioInputStream(pcmConvertedAudioInputStream);
        if (pcmConvertedAudioInputStream == null) {
            return;
        }
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, pcmConvertedAudioInputStream.getFormat());
        if (!AudioSystem.isLineSupported(info)) {
            return;
        }
        DataPusher dataPusher = new DataPusher((SourceDataLine)AudioSystem.getLine(info), pcmConvertedAudioInputStream);
        this.infos.addElement(new AudioDevice.Info(null, inputStream, dataPusher));
        dataPusher.start();
        return;
    }
    
    private synchronized void startMidi(InputStream sequence, InputStream inputStream) throws InvalidMidiDataException, MidiUnavailableException {
        Sequencer sequencer = MidiSystem.getSequencer();
        sequencer.open();
        try {
            sequencer.setSequence(sequence);
        }
        catch (IOException ex) {
            throw new InvalidMidiDataException(ex.getMessage());
        }
        AudioDevice.Info obj = new AudioDevice.Info(sequencer, inputStream, null);
        this.infos.addElement(obj);
        sequencer.addMetaEventListener(obj);
        sequencer.start();
    }
    
    public synchronized void openChannel(InputStream inputStream) {
        if (this.DEBUG) {
            System.out.println("AudioDevice: openChannel");
            System.out.println("input stream =" + inputStream);
        }
        int i = 0;
        while (i < this.infos.size()) {
            if (((AudioDevice.Info)this.infos.elementAt(i)).in == inputStream) {
                return;
            }
            ++i;
        }
        if ((inputStream instanceof AudioStream)) {
            if (((AudioStream)inputStream).midiformat != null) {
                try {
                    this.startMidi(((AudioStream)inputStream).stream, inputStream);
                }
                catch (Exception ex12) {
                    return;
                }
            }
            else if (((AudioStream)inputStream).ais != null) {
                try {
                    this.startSampled(((AudioStream)inputStream).ais, inputStream);
                }
                catch (Exception ex11) {
                    return;
                }
            }
        }
        else if ((inputStream instanceof AudioDataStream)) {
            if ((inputStream instanceof ContinuousAudioDataStream)) {
                try {
                    this.startSampled(new AudioInputStream(inputStream, ((AudioDataStream)inputStream).getAudioData().format, -1L), inputStream);
                }
                catch (Exception ex10) {
                    return;
                }
            }
            else {
                try {
                    this.startSampled(new AudioInputStream(inputStream, ((AudioDataStream)inputStream).getAudioData().format, (long)((AudioDataStream)inputStream).getAudioData().buffer.length), inputStream);
                }
                catch (Exception ex9) {
                    return;
                }
            }
        }
        else {
            BufferedInputStream stream = new BufferedInputStream(inputStream, 1024);
            try {
                AudioInputStream audioInputStream;
                try {
                    audioInputStream = AudioSystem.getAudioInputStream(stream);
                }
                catch (IOException ex) {
                    return;
                }
                this.startSampled(audioInputStream, inputStream);
            }
            catch (UnsupportedAudioFileException ex2) {
                try {
                    try {
                        MidiSystem.getMidiFileFormat(stream);
                    }
                    catch (IOException ex3) {
                        return;
                    }
                    this.startMidi(stream, inputStream);
                }
                catch (InvalidMidiDataException ex4) {
                    AudioFormat format = new AudioFormat(AudioFormat.Encoding.ULAW, 8000.0f, 8, 1, 1, 8000.0f, true);
                    try {
                        this.startSampled(new AudioInputStream(stream, format, -1L), inputStream);
                    }
                    catch (UnsupportedAudioFileException ex5) {
                        return;
                    }
                    catch (LineUnavailableException ex6) {
                        return;
                    }
                }
                catch (MidiUnavailableException ex7) {
                    return;
                }
            }
            catch (LineUnavailableException ex8) {
                return;
            }
        }
        this.notify();
    }
    
    public synchronized void closeChannel(InputStream inputStream) {
        if (this.DEBUG) {
            System.out.println("AudioDevice.closeChannel");
        }
        if (inputStream == null) {
            return;
        }
        for (int i = 0; i < this.infos.size(); ++i) {
            AudioDevice.Info info = (AudioDevice.Info)this.infos.elementAt(i);
            if (info.in == inputStream) {
                if (info.sequencer != null) {
                    info.sequencer.stop();
                    this.infos.removeElement(info);
                }
                else if (info.datapusher != null) {
                    info.datapusher.stop();
                    this.infos.removeElement(info);
                }
            }
        }
        this.notify();
        return;
    }
    
    public synchronized void open() {
    }
    
    public synchronized void close() {
    }
    
    public void play() {
        if (this.DEBUG) {
            System.out.println("exiting play()");
        }
    }
    
    public synchronized void closeStreams() {
        for (int i = 0; i < this.infos.size(); ++i) {
            AudioDevice.Info info = (AudioDevice.Info)this.infos.elementAt(i);
            if (info.sequencer != null) {
                info.sequencer.stop();
                info.sequencer.close();
                this.infos.removeElement(info);
            }
            else if (info.datapusher != null) {
                info.datapusher.stop();
                this.infos.removeElement(info);
            }
        }
        if (this.DEBUG) {
            System.err.println("Audio Device: Streams all closed.");
        }
        this.clipStreams = new Hashtable();
        this.infos = new Vector();
    }
    
    public int openChannels() {
        return this.infos.size();
    }
    
    void setVerbose(boolean debug) {
        this.DEBUG = debug;
    }
    
    static {
        device = new AudioDevice();
    }
    
    final class Info implements MetaEventListener
    {
        final Sequencer sequencer;
        final InputStream in;
        final DataPusher datapusher;
        
        Info(Sequencer sequencer, InputStream in, DataPusher datapusher) {
            this.sequencer = sequencer;
            this.in = in;
            this.datapusher = datapusher;
        }
        
        @Override
        public void meta(MetaMessage metaMessage) {
            if (metaMessage.getType() == 47 && this.sequencer != null) {
                this.sequencer.close();
            }
        }
    }
}
